@@ -1,12 +1,13 @@
-from enum import Enum
-from .import linear_model_pb2
-import logging
 from . import constraints
+from . import linear_model_pb2
+from . import mp_model
 from . import optilab_pb2
 from . import optimizer_model_pb2
-from . import mp_model
-from . import variables
 from . import parallel_model
+from . import variables
+
+from enum import Enum
+import logging
 import string
 import sys
 
@@ -29,17 +30,15 @@ class OptimizerPackageType(Enum):
     SCIP = 4
     GLPK = 5
 
-
 def infinity():
     return sys.float_info.max
 
-
 class OptimizerMPModel(parallel_model.ParallelModel):
     """OptiLab Mathematical Programming (MP) model
     solved by back-end optimizers"""
 
-    variable_limit = 100
-    constraint_limit = 100
+    variable_limit = 100000
+    constraint_limit = 100000
     mp_model = None
     mp_model_file_path = ""
     model_type = None
@@ -64,6 +63,10 @@ class OptimizerMPModel(parallel_model.ParallelModel):
         self.model_name = name
         if name.strip():
             self.mp_model = mp_model.OptMPModel(name)
+
+        # Default type is protobuf
+        self.model_type = linear_model_pb2.LinearModelSpecProto.PROTOBUF
+
         # Default package is SCIP
         self.package_type = OptimizerPackageType.SCIP
 
@@ -71,6 +74,7 @@ class OptimizerMPModel(parallel_model.ParallelModel):
         if optilab_reply_message.details.Is(
                 optilab_pb2.OptimizerSolutionRep.DESCRIPTOR):
             # JSON solutions are printed verbatim on the screen
+            # @note DEPRECATED
             sol_msg = optilab_pb2.OptimizerSolutionRep()
             optilab_reply_message.details.Unpack(sol_msg)
             print(sol_msg.solution)
@@ -79,6 +83,10 @@ class OptimizerMPModel(parallel_model.ParallelModel):
             sol_proto = linear_model_pb2.LinearModelSolutionProto()
             optilab_reply_message.details.Unpack(sol_proto)
             self.upload_proto_solution(sol_proto)
+        else:
+            msg = "OptimizerMPModel - received an unrecognized back-end message"
+            logging.error(err_msg)
+            print(msg)
 
     def get_model_status(self):
         return self.status_to_string(self.model_status)
@@ -93,7 +101,11 @@ class OptimizerMPModel(parallel_model.ParallelModel):
             return linear_model_pb2.LinearModelSolutionStatusProto.Name(status)
 
     def build_model_from_solution(self, linear_model_solution_proto):
-        self.mp_model = OptMPModel.OptMPModel(self.model_name)
+        self.mp_model = mp_model.OptMPModel(self.model_name)
+
+        # Set the objective value
+        self.mp_model.set_objective_value(
+            linear_model_solution_proto.objective_value)
 
         # Set variable values
         if self.model_status == linear_model_pb2.LinearModelSolutionStatusProto.Value('SOLVER_FEASIBLE') or \
@@ -106,16 +118,19 @@ class OptimizerMPModel(parallel_model.ParallelModel):
         # Set the model's status
         self.model_status = linear_model_solution_proto.status
 
-        # Set the objective value
-        self.mp_model.set_objective_value(
-            linear_model_solution_proto.objective_value)
+        if self.mp_model is None:
+            self.build_model_from_solution(linear_model_solution_proto)
+        else:
+            # Set the objective value
+            self.mp_model.set_objective_value(
+                linear_model_solution_proto.objective_value)
 
-        # Set variable values
-        if self.model_status == linear_model_pb2.LinearModelSolutionStatusProto.Value('SOLVER_FEASIBLE') or \
+            # Set variable values
+            if self.model_status == linear_model_pb2.LinearModelSolutionStatusProto.Value('SOLVER_FEASIBLE') or \
                 self.model_status == linear_model_pb2.LinearModelSolutionStatusProto.Value('SOLVER_OPTIMAL'):
-            for proto_var in linear_model_solution_proto.variable_assign:
-                var = self.mp_model.get_variable_from_name(proto_var.var_name)
-                var.set_solution_value(proto_var.var_value[0])
+                for proto_var in linear_model_solution_proto.variable_assign:
+                    var = self.mp_model.get_variable_from_name(proto_var.var_name)
+                    var.set_solution_value(proto_var.var_value[0])
 
         # Notify the user about model upload
         print("MIPModel - solution uploaded")
@@ -127,7 +142,6 @@ class OptimizerMPModel(parallel_model.ParallelModel):
                 type(model_type)
             logging.error(err_msg)
             raise Exception(err_msg)
-        self.model_type = model_type
         if model_type == OptimizerModelType.MIP:
             self.model_user_type = 0b11
         elif model_type == OptimizerModelType.IP:
@@ -184,13 +198,24 @@ class OptimizerMPModel(parallel_model.ParallelModel):
         self.mp_model_file_path = None
         self.mp_model = OptMPModel.OptMPModel(name)
 
-    def load_model_from_file(self, file_path, name):
+    def load_model_from_file(self, file_path, name, model_type = linear_model_pb2.LinearModelSpecProto.MPS):
+        """Loads an MP model from a file.
+        The supported formats are:
+        - mps
+        """
         if not name.strip():
             err_msg = "OptimizerMPModel - LoadFromFile: empty model name"
             logging.error(err_msg)
             raise Exception(err_msg)
         self.mp_model = None
         self.mp_model_file_path = file_path
+        self.model_type = model_type
+
+    def load_mps_model(self, file_path, name):
+        self.load_model_from_file(file_path, name, linear_model_pb2.LinearModelSpecProto.MPS)
+
+    def load_csv_model(self, file_path, name):
+        self.load_model_from_file(file_path, name, linear_model_pb2.LinearModelSpecProto.CSV)
 
     def clear(self):
         """Clears the model"""
@@ -379,6 +404,9 @@ class OptimizerMPModel(parallel_model.ParallelModel):
         # Set model name
         optimizer_model.linear_model.model_id = self.model_name
 
+        # Set model type
+        optimizer_model.linear_model.model_format_type = self.model_type
+
         # Set model and package type
         m_type = self.get_model_type()
         if m_type is OptimizerModelType.MIP:
